# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock, patch

from mattermost_notify.errors import MattermostNotifyError
from mattermost_notify.post import post


class PostTestCase(unittest.TestCase):
    @patch("mattermost_notify.post.httpx.post", autospec=True)
    def test_success(self, post_mock: MagicMock):
        response = post_mock.return_value
        response.is_success = True

        post("https://some.mattermost.url", "FooChannel", "Some Message")

        post_mock.assert_called_once_with(
            url="https://some.mattermost.url",
            json={"channel": "FooChannel", "text": "Some Message"},
        )

    @patch("mattermost_notify.post.httpx.post", autospec=True)
    def test_failure(self, post_mock: MagicMock):
        response = post_mock.return_value
        response.is_success = False
        response.status_code = 500

        with self.assertRaisesRegex(
            MattermostNotifyError,
            "Failed to post on Mattermost. HTTP status was 500",
        ):
            post("https://some.mattermost.url", "FooChannel", "Some Message")

        post_mock.assert_called_once_with(
            url="https://some.mattermost.url",
            json={"channel": "FooChannel", "text": "Some Message"},
        )



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import httpx
from pontos.typing import SupportsStr

from mattermost_notify.errors import MattermostNotifyError


def post(url: str, channel: str, text: SupportsStr) -> None:
    response = httpx.post(url=url, json={"channel": channel, "text": text})
    if not response.is_success:
        raise MattermostNotifyError(
            "Failed to post on Mattermost. HTTP status was "
            f"{response.status_code}"
        )



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from enum import Enum


class Status(Enum):
    SUCCESS = ":white_check_mark: success"
    FAILURE = ":x: failure"
    UNKNOWN = ":grey_question: unknown"
    CANCELLED = ":no_entry_sign: canceled"

    def __str__(self):
        return self.name



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


class MattermostNotifyError(Exception):
    """Base error for all errors originating from mattermost-notify"""



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from mattermost_notify.parser import parse_args


class ParseArgsTestCase(unittest.TestCase):
    def test_defaults(self):
        parsed_args = parse_args(["www.url.de", "channel"])

        self.assertEqual(parsed_args.url, "www.url.de")
        self.assertEqual(parsed_args.channel, "channel")

        self.assertFalse(parsed_args.short)
        self.assertEqual(parsed_args.status, "SUCCESS")
        self.assertIsNone(parsed_args.repository)
        self.assertIsNone(parsed_args.branch)
        self.assertIsNone(parsed_args.workflow)
        self.assertIsNone(parsed_args.workflow_name)
        self.assertIsNone(parsed_args.commit)
        self.assertIsNone(parsed_args.free)
        self.assertIsNone(parsed_args.highlight)

    def test_fail_argument_parsing(self):
        with self.assertRaises(SystemExit):
            parse_args(["-s"])

    def test_parse_status(self):
        parsed_args = parse_args(
            [
                "www.url.de",
                "channel",
                "-S",
                "failure",
            ]
        )
        self.assertEqual(parsed_args.status, "failure")

    def test_parse_short(self):
        parsed_args = parse_args(
            [
                "www.url.de",
                "channel",
                "--short",
            ]
        )
        self.assertTrue(parsed_args.short)

        parsed_args = parse_args(
            [
                "www.url.de",
                "channel",
                "-s",
            ]
        )
        self.assertTrue(parsed_args.short)

    def test_parse_repository(self):
        parsed_args = parse_args(["www.url.de", "channel", "-r", "foo/bar"])
        self.assertEqual(parsed_args.repository, "foo/bar")

        parsed_args = parse_args(
            ["www.url.de", "channel", "--repository", "foo/bar"]
        )
        self.assertEqual(parsed_args.repository, "foo/bar")

    def test_parse_branch(self):
        parsed_args = parse_args(["www.url.de", "channel", "-b", "foo"])
        self.assertEqual(parsed_args.branch, "foo")

        parsed_args = parse_args(["www.url.de", "channel", "--branch", "foo"])
        self.assertEqual(parsed_args.branch, "foo")

    def test_parse_workflow(self):
        parsed_args = parse_args(["www.url.de", "channel", "-w", "w1"])
        self.assertEqual(parsed_args.workflow, "w1")

        parsed_args = parse_args(["www.url.de", "channel", "--workflow", "w1"])
        self.assertEqual(parsed_args.workflow, "w1")

    def test_parse_workflow_name(self):
        parsed_args = parse_args(["www.url.de", "channel", "-n", "foo"])
        self.assertEqual(parsed_args.workflow_name, "foo")

        parsed_args = parse_args(
            ["www.url.de", "channel", "--workflow_name", "foo"]
        )
        self.assertEqual(parsed_args.workflow_name, "foo")

    def test_parse_commit(self):
        parsed_args = parse_args(["www.url.de", "channel", "--commit", "1234"])
        self.assertEqual(parsed_args.commit, "1234")

    def test_parse_commit_message(self):
        parsed_args = parse_args(
            ["www.url.de", "channel", "--commit-message", "foo bar"]
        )
        self.assertEqual(parsed_args.commit_message, "foo bar")

    def test_parse_free(self):
        parsed_args = parse_args(
            ["www.url.de", "channel", "--free", "lorem ipsum"]
        )
        self.assertEqual(parsed_args.free, "lorem ipsum")

    def test_parse_highlight(self):
        parsed_args = parse_args(
            [
                "www.url.de",
                "channel",
                "--highlight",
                "user1",
                "user2",
            ]
        )
        self.assertEqual(parsed_args.highlight, ["user1", "user2"])



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# pylint: disable=invalid-name

import json
import os
from pathlib import Path
from typing import Any, Optional

from pontos.git import Git
from pontos.terminal import RichTerminal

from mattermost_notify.errors import MattermostNotifyError
from mattermost_notify.parser import parse_args
from mattermost_notify.post import post
from mattermost_notify.status import Status

LONG_TEMPLATE = (
    "#### Status: {status}\n\n"
    "| Workflow | {workflow} |\n"
    "| --- | --- |\n"
    "| Repository (branch) | {repository} ({branch}) |\n"
    "| Related commit | {commit} |\n\n"
    "{highlight}"
)

SHORT_TEMPLATE = "{status}: {workflow} ({commit}) in {repository} ({branch})"

DEFAULT_GIT = "https://github.com"


def linker(name: str, url: Optional[str] = None) -> str:
    # create a markdown link
    return f"[{name}]({url})" if url else name


def get_github_event_json() -> dict[str, Any]:
    github_event_path = os.environ.get("GITHUB_EVENT_PATH")

    if not github_event_path:
        return {}

    json_path = Path(github_event_path)

    try:
        with json_path.open("r", encoding="utf-8") as f:
            return json.load(f)
    except FileNotFoundError:
        raise MattermostNotifyError("Could not find GitHub Event JSON file.")
    except json.JSONDecodeError:
        raise MattermostNotifyError("Could not decode the JSON object.")


def fill_template(
    *,
    short: bool = False,
    highlight: Optional[list[str]] = None,
    commit: Optional[str] = None,
    commit_message: Optional[str] = None,
    branch: Optional[str] = None,
    repository: Optional[str] = None,
    status: Optional[str] = None,
    workflow_id: Optional[str] = None,
    workflow_name: Optional[str] = None,
) -> str:
    template = SHORT_TEMPLATE if short else LONG_TEMPLATE

    # try to get information from the GiTHUB_EVENT json
    event = get_github_event_json()
    workflow_info: dict[str, Any] = event.get("workflow_run", {})

    status = status if status else workflow_info.get("conclusion")
    workflow_status = Status[status.upper()] if status else Status.UNKNOWN

    used_workflow_name: str = (
        workflow_name if workflow_name else workflow_info.get("name", "")
    )
    used_workflow_id = (
        workflow_id if workflow_id else workflow_info.get("workflow_id", "")
    )

    head_repo: dict[str, Any] = workflow_info.get("head_repository", {})
    repository = repository if repository else head_repo.get("full_name", "")
    repository_url = (
        f"{DEFAULT_GIT}/{repository}"
        if repository
        else head_repo.get("html_url", "")
    )

    used_branch: str = (
        branch if branch else workflow_info.get("head_branch", "")
    )
    branch_url = f"{repository_url}/tree/{used_branch}"

    workflow_url = (
        f"{repository_url}/actions/runs/{used_workflow_id}"
        if repository
        else workflow_info.get("html_url", "")
    )

    head_commit = workflow_info.get("head_commit", {})

    if commit:
        commit_url = f"{repository_url}/commit/{commit}"

        if not commit_message:
            commit_message = Git().show(
                format="format:%s", patch=False, objects=commit  # type: ignore[assignment] # noqa: E501
            )
    else:
        commit_url = f'{repository_url}/commit/{head_commit.get("id", "")}'

        if not commit_message:
            commit_message = head_commit.get("message", "").split("\n", 1)[0]

    highlight_str = ""
    if highlight and workflow_status is not Status.SUCCESS:
        highlight_str = "".join([f"@{h}\n" for h in highlight])

    return template.format(
        status=workflow_status.value,
        workflow=linker(used_workflow_name, workflow_url),
        repository=linker(repository, repository_url),
        branch=linker(used_branch, branch_url),
        commit=linker(commit_message, commit_url),  # type: ignore[arg-type]
        highlight=highlight_str,
    )


def main() -> None:
    parsed_args = parse_args()

    term = RichTerminal()

    try:
        if not parsed_args.free:
            body = fill_template(
                highlight=parsed_args.highlight,
                short=parsed_args.short,
                branch=parsed_args.branch,
                commit=parsed_args.commit,
                repository=parsed_args.repository,
                status=parsed_args.status,
                workflow_id=parsed_args.workflow,
                workflow_name=parsed_args.workflow_name,
            )
            post(parsed_args.url, parsed_args.channel, body)
        else:
            post(parsed_args.url, parsed_args.channel, parsed_args.free)

        term.ok(
            "Successfully posted on Mattermost channel "
            f"{parsed_args.channel}"
        )
    except MattermostNotifyError as e:
        term.error(f"❌ Error: {e}")


if __name__ == "__main__":
    main()



# SPDX-FileCopyrightText: 2022-2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

# ruff: noqa: E501

import unittest
from unittest.mock import MagicMock, patch

from mattermost_notify.git import fill_template, linker
from mattermost_notify.status import Status


class LinkerTestCase(unittest.TestCase):
    def test_linker(self):
        self.assertEqual(linker("foo", "www.foo.com"), "[foo](www.foo.com)")

    def test_no_url(self):
        self.assertEqual(linker("foo"), "foo")
        self.assertEqual(linker("foo", None), "foo")
        self.assertEqual(linker("foo", ""), "foo")


class FillTemplateTestCase(unittest.TestCase):
    def test_success_no_highlight(self):
        actual = fill_template(
            highlight=["user1", "user2"],
            status=Status.SUCCESS.name,
        )
        expected = """#### Status: :white_check_mark: success

| Workflow |  |
| --- | --- |
| Repository (branch) |  ([](/tree/)) |
| Related commit | [](/commit/) |

"""
        self.assertEqual(expected, actual)

    def test_failure_highlight(self):
        actual = fill_template(
            highlight=["user1", "user2"],
            status=Status.FAILURE.name,
        )
        expected = """#### Status: :x: failure

| Workflow |  |
| --- | --- |
| Repository (branch) |  ([](/tree/)) |
| Related commit | [](/commit/) |

@user1
@user2
"""
        self.assertEqual(expected, actual)

    def test_short_template(self):
        actual = fill_template(
            short=True,
            status=Status.SUCCESS.name,
            workflow_name="SomeWorkflow",
            workflow_id="w1",
            commit="12345",
            commit_message="Add foo",
            repository="foo/bar",
            branch="main",
        )
        expected = (
            ":white_check_mark: success: [SomeWorkflow](https://github.com/foo/bar/actions/runs/w1) "
            "([Add foo](https://github.com/foo/bar/commit/12345)) in "
            "[foo/bar](https://github.com/foo/bar) ([main](https://github.com/foo/bar/tree/main))"
        )
        self.assertEqual(expected, actual)

    def test_template(self):
        actual = fill_template(
            short=False,
            status=Status.SUCCESS.name,
            workflow_name="SomeWorkflow",
            workflow_id="w1",
            commit="12345",
            commit_message="Add foo",
            repository="foo/bar",
            branch="main",
        )
        expected = """#### Status: :white_check_mark: success

| Workflow | [SomeWorkflow](https://github.com/foo/bar/actions/runs/w1) |
| --- | --- |
| Repository (branch) | [foo/bar](https://github.com/foo/bar) ([main](https://github.com/foo/bar/tree/main)) |
| Related commit | [Add foo](https://github.com/foo/bar/commit/12345) |

"""
        self.assertEqual(expected, actual)

    @patch("mattermost_notify.git.get_github_event_json")
    def test_template_data_from_github_event(self, mock: MagicMock):
        event = {
            "workflow_run": {
                "conclusion": Status.SUCCESS.name,
                "name": "SomeWorkflow",
                "head_repository": {
                    "full_name": "foo/bar",
                    "html_url": "https://github.com/foo/bar",
                },
                "head_branch": "main",
                "head_commit": {"id": "12345", "message": "Add foo"},
                "workflow_id": "w1",
            }
        }
        mock.return_value = event

        actual = fill_template()
        expected = """#### Status: :white_check_mark: success

| Workflow | [SomeWorkflow](https://github.com/foo/bar/actions/runs/w1) |
| --- | --- |
| Repository (branch) | [foo/bar](https://github.com/foo/bar) ([main](https://github.com/foo/bar/tree/main)) |
| Related commit | [Add foo](https://github.com/foo/bar/commit/12345) |

"""
        self.assertEqual(expected, actual)



# SPDX-FileCopyrightText: 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import ArgumentParser, Namespace

from mattermost_notify.status import Status


def parse_args(args=None) -> Namespace:
    parser = ArgumentParser()

    parser.add_argument(
        "url",
        help="Mattermost (WEBHOOK) URL",
        type=str,
    )

    parser.add_argument(
        "channel",
        type=str,
        help="Mattermost Channel",
    )

    parser.add_argument(
        "-s",
        "--short",
        action="store_true",
        help="Send a short single line message",
    )

    parser.add_argument(
        "-S",
        "--status",
        type=str,
        choices=["success", "failure"],
        default=Status.SUCCESS.name,
        help="Status of Job",
    )

    parser.add_argument(
        "-r", "--repository", type=str, help="git repository name (orga/repo)"
    )

    parser.add_argument("-b", "--branch", type=str, help="git branch")

    parser.add_argument(
        "-w", "--workflow", type=str, help="hash/ID of the workflow"
    )

    parser.add_argument(
        "-n", "--workflow_name", type=str, help="name of the workflow"
    )

    parser.add_argument("--commit", help="Commit ID to use")
    parser.add_argument("--commit-message", help="Commit Message to use")

    parser.add_argument(
        "--free",
        type=str,
        help="Print a free-text message to the given channel",
    )

    parser.add_argument(
        "--highlight",
        nargs="+",
        help="List of persons to highlight in the channel",
    )

    return parser.parse_args(args=args)
